"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = exports.HyperServer = void 0;
const helmet_1 = __importDefault(require("helmet"));
const compression_1 = __importDefault(require("compression"));
const hypernight_1 = require("@hyperstackjs/hypernight");
const body_parser_1 = require("body-parser");
const express_pino_logger_1 = __importDefault(require("express-pino-logger"));
const pino_1 = __importDefault(require("pino"));
const debug_1 = __importDefault(require("debug"));
const index_sender_1 = __importDefault(require("./index-sender"));
const not_found_1 = __importDefault(require("./not-found"));
const set_request_id_1 = __importDefault(require("./set-request-id"));
const error_handler_1 = __importDefault(require("./error-handler"));
const debug = (0, debug_1.default)('hyperserver');
const getOrCreateLogger = (opts) => {
    if (opts === false) {
        return null;
    }
    if (opts && opts !== true) {
        return opts;
    }
    debug('creating logger');
    return (0, pino_1.default)();
};
const getOrCreateMiddleware = (logger, opts) => {
    if (opts === false) {
        return null;
    }
    if (opts && opts !== true) {
        return opts;
    }
    debug('creating own logger middleware');
    return (0, express_pino_logger_1.default)({ logger: logger }); // workaround: e-p-l needs pino 7, while we use pino 8 and types conflict
};
const defaults = {
    helmet: undefined,
    json: { limit: '5mb' },
    urlencoded: { limit: '5mb', extended: true },
    requestIdOpts: { mode: 'passthrough-or-generate' },
};
const isDefaultSetting = (s) => s === true || s === undefined;
class HyperServer extends hypernight_1.Server {
    constructor(controllers, opts) {
        const { cookieParserMiddleware, forceHttpsMiddleware, staticMiddleware, } = require('../middleware'); // eslint-disable-line @typescript-eslint/no-var-requires
        super(process.env.NODE_ENV === 'development'); // setting showLogs to true
        const { staticFolder, indexCatchAll, forceHttps, gzip, cookieSecret, logging, helmet: helmetOpts, json: jsonOpts, urlencoded: urlencodedOpts, requestId: requestIdOpts, sendValidationErrors, beforeControllers, afterControllers, beforeMiddleware, afterMiddleware, } = opts;
        const logger = getOrCreateLogger(logging && logging.logger);
        const middleware = getOrCreateMiddleware(logger, logging && logging.middleware);
        if (beforeMiddleware) {
            beforeMiddleware(this.app);
        }
        if (requestIdOpts !== false) {
            const res = isDefaultSetting(requestIdOpts)
                ? defaults.requestIdOpts
                : requestIdOpts;
            this.app.use((0, set_request_id_1.default)(res));
        }
        // logging middleware set up
        if (middleware) {
            this.app.use(middleware);
            this.app.use((req, res, next) => {
                // take the pino middleware logger (available in .log) which is enriched with request data,
                // or take a 'dry' logger created here
                ;
                req.logger = req.log || logger;
                if (next) {
                    next();
                }
            });
        }
        if (forceHttps) {
            this.app.use(forceHttpsMiddleware);
        }
        if (gzip !== false) {
            const res = isDefaultSetting(gzip) ? undefined : gzip;
            debug('gzip: %o', res);
            this.app.use((0, compression_1.default)(res));
        }
        if (helmetOpts !== false) {
            const res = isDefaultSetting(helmetOpts) ? defaults.helmet : helmetOpts;
            debug('helmet: %o', res);
            this.app.use((0, helmet_1.default)(res));
        }
        if (jsonOpts !== false) {
            const res = isDefaultSetting(jsonOpts) ? defaults.json : jsonOpts;
            debug('json: %o', res);
            this.app.use((0, body_parser_1.json)(res));
        }
        if (urlencodedOpts !== false) {
            const res = isDefaultSetting(urlencodedOpts)
                ? defaults.urlencoded
                : urlencodedOpts;
            debug('urlencoded: %o', res);
            this.app.use((0, body_parser_1.urlencoded)(res));
        }
        if (cookieSecret) {
            this.app.use(cookieParserMiddleware(cookieSecret));
        }
        if (staticFolder) {
            this.app.use(staticMiddleware(staticFolder));
        }
        if (beforeControllers) {
            beforeControllers(this.app);
        }
        super.addControllers(controllers);
        if (afterControllers) {
            afterControllers(this.app);
        }
        if (staticFolder && indexCatchAll) {
            this.app.get('*', (0, index_sender_1.default)(staticFolder));
        }
        this.app.use(not_found_1.default);
        // give the error handler a logger of its own, guaranteed to exist, in case
        // things go SO bad and a request doesn't even get to have a logger in req.logger
        this.app.use((0, error_handler_1.default)({ logger, sendValidationErrors: !!sendValidationErrors })); // logger can be null if user said 'logger: false'
        if (afterMiddleware) {
            afterMiddleware(this.app);
        }
        ;
        this.app.__opts = opts; // hide for metadata about our app, to list out in printers
    }
}
exports.HyperServer = HyperServer;
const createServer = ({ controllers, opts = {}, }) => {
    const server = new HyperServer(controllers.map((K) => new K()), opts);
    return server.app;
};
exports.createServer = createServer;
