"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.requires = exports.permits = exports.ParseBody = exports.ValidateBody = void 0;
const ajv_1 = __importDefault(require("ajv"));
const lodash_1 = __importDefault(require("lodash"));
const hypernight_1 = require("@hyperstackjs/hypernight");
const errors_1 = require("./errors");
// strong params, permit only what's in schema
const permits = (schema) => schema.parse.bind(schema);
exports.permits = permits;
// actionparams, permit and require all that's in schema
const requires = (schema) => {
    const s = schema.strict();
    return (d) => {
        const result = s.safeParse(d);
        if (!result.success) {
            throw new errors_1.ValidationError(result.error.issues);
        }
        return result.data;
    };
};
exports.requires = requires;
// decorator, powered by zod. validates + parses and assigns parsed data to body
function ParseBody(schema) {
    function parseBody(req, res, next) {
        const { success, data, error } = schema.safeParse(req.body);
        if (!success) {
            throw new errors_1.ValidationError(error.issues);
        }
        else {
            req.body = data;
        }
        next();
    }
    parseBody.params = { schema: { zod: schema } };
    return (0, hypernight_1.Middleware)(parseBody);
}
exports.ParseBody = ParseBody;
// decorator, powered by ajv, only validates.
function ValidateBody(schema, customValidations = {}) {
    const ajv = new ajv_1.default();
    lodash_1.default.toPairs(customValidations).forEach(([validationName, validationSchema]) => {
        ajv.addKeyword(validationName, validationSchema);
    });
    function validateBody(req, res, next) {
        const validator = ajv.compile(schema); // pay compilation because we dont want to share instance
        if (!validator(req.body)) {
            throw new errors_1.ValidationError(validator.errors);
        }
        next();
    }
    return (0, hypernight_1.Middleware)(validateBody);
}
exports.ValidateBody = ValidateBody;
